{This document is Copyright  1997-98 Project JEDI.
Visit the JEDI home page at: http://www.delphi-jedi.org/}

{---------------------------------------------------------
Copyright (c) 1996-98 Massimo Maria Ghisalberti
CopyRight (c) 1996-98 MAx!
CopyRight (c) 1996-98 Objects Built for you! (O.B.you!)
Internet EMail: obyou@tin.it

name:           InitUser.dpr
date:           08/18/1998
version:        1 beta revision 1.

note: use at own risk.
      If you use this code, please mention O.B.you!
      somewhere in your program
---------------------------------------------------------
REVISION:
  1.1 Eliminated SysUtils from uses clause, size of exe
      now is 18kb with Delphi4, 17 with Delphi3/2, (old was 41kb).
  1.0 first version.
---------------------------------------------------------

Initialize Client
------------------

The INITUSER sample sets up the default cryptographic client. This sample
(or its equivalent) must be run prior to running the other cryptography
samples, so that they will work correctly.

The following operations are performed:

1. A default key container is created.
2. A digital signature key pair is created within the key container.
3. A key exchange key pair is created within the key container.

These operations only need to be done once, unless the operating system
is reinstalled. If the default key container and key pairs have already
been created, this sample performs no operations.

The INITUSER sample uses the following CryptoAPI functions:

    CryptAcquireContext
    CryptGetProvParam
    CryptGenKey
    CryptGetUserKey
    CryptDestroyKey
    CryptReleaseContext
---------------------------------------------------------}

program InitUser;
{$APPTYPE CONSOLE}

uses Windows, WinCrypt;

const
  MESSAGE1 = '-------------------------------------------------------------'+#13#10+
             'InitUser ver. 1.1'                                            +#13#10+
             '-------------------------------------------------------------'+#13#10+
                                                                             #13#10+
             'This utility create a new default '                           +#13#10+
             'Key Container for a current user. '                           +#13#10+
             '-------------------------------------------------------------'+#13#10;

 MESSAGE2 =  'key Container for Current User    '                           +#13#10+
             'succesfully created.';

 MESSAGE3 =  '----------------------------------'                           +#13#10+
             'Some errors occurs during creation'                           +#13#10+
             'of key Container for Current User.';
             
 CRLF     = #13#10;
             
function FormatMsgToCon(lpOut :PChar; arglist :va_list ):DWORD;
var
  OutMsgBuff :array[0..255] of char;
begin
  Result := wvsprintf(@OutMsgBuff,lpOut,arglist);
  Write(OutMsgBuff);
end;             

function StrAlloc(Size: Cardinal): PChar;
begin
  Inc(Size, SizeOf(Cardinal));
  GetMem(Result, Size);
  Cardinal(Pointer(Result)^) := Size;
  Inc(Result, SizeOf(Cardinal));
end;

procedure StrDispose(Str: PChar);
begin
  if Str <> nil then
  begin
    Dec(Str, SizeOf(Cardinal));
    FreeMem(Str, Cardinal(Pointer(Str)^));
  end;
end;

function InitUserForCSP :boolean;
var
  szUserName :PChar;
  dwUserNameLen :DWORD;
  hProv :HCRYPTPROV;
  hKey :HCRYPTKEY;
  args :array[0..0] of pointer;
begin
  Result := False;

  hProv := 0;
  hKey := 0;

  dwUserNameLen := 100;
  szUserName := StrAlloc(dwUserNameLen);
  {Try to acquire an handle to CSP.}
  if not CryptAcquireContext(@hProv,nil,nil,PROV_RSA_FULL,0) then
    {Registry setting for CSP not exist...}
    begin
      {Create registry setting with parameter: CRYPT_NEWKEYSET}
      {If error then exit from routine.}
      if not CryptAcquireContext(@hProv,nil,nil,PROV_RSA_FULL,CRYPT_NEWKEYSET) then
        begin
          //WriteLn(Format('Error num: %d on creating key container!',[GetLastError]));
          args[0] := Pointer(GetLastError);
          FormatMsgToCon('Error num: %d on creating key container!'+CRLF, @args);
          Exit;
        end;
      {Try to acquire default key container.}
      if not CryptGetProvParam(hProv,PP_CONTAINER,PBYTE(szUserName),@dwUserNameLen,0) then
        {Error.}
        szUserName[0] := char(0);
      WriteLn('Key Container Name: '+szUserName);
    end;
  {Try to acquire an Handle for signature.}
  if not CryptGetUserKey(hProv,AT_SIGNATURE,@hKey) then
    begin
      if GetLastError = NTE_NO_KEY then
        begin
          {Build signature.}
          if not CryptGenKey(hProv,AT_SIGNATURE,0,@hKey) then
            begin
              //WriteLn(Format('Error num: %d during CryptGenKey',[GetLastError]));
              args[0] := Pointer(GetLastError);
              FormatMsgToCon('Error num: %d during CryptGenKey'+CRLF, @args);
              Exit;
            end
          else
            CryptDestroyKey(hKey);
        end
      else
        begin
          //WriteLn(Format('Error num: %d during CryptGetUserKey',[GetLastError]));
          args[0] := Pointer(GetLastError);
          FormatMsgToCon('Error num: %d during CryptGetUserKey'+CRLF, @args);
          Exit;
        end;
    end;
  {Try to acquire handle for exporting key.}
  if not CryptGetUserKey(hProv,AT_KEYEXCHANGE,@hKey) then
    begin
      if GetLastError = NTE_NO_KEY then
        begin
          {Build exchange key.}
          if not CryptGenKey(hProv,AT_KEYEXCHANGE,0,@hKey) then
            begin
              //WriteLn(Format('Error num: %d during CryptGenKey',[GetLastError]));
              args[0] := Pointer(GetLastError);
              FormatMsgToCon('Error num: %d during CryptGenKey'+CRLF, @args);
              Exit;
            end
          else
            CryptDestroyKey(hKey);
        end
      else
        begin
          //WriteLn(Format('Error num: %d during CryptGetUserKey',[GetLastError]));
          args[0] := Pointer(GetLastError);
          FormatMsgToCon('Error num: %d during CryptGetUserKey'+CRLF, @args);
          Exit;
        end;
    end;
  CryptReleaseContext(hProv,0);
  StrDispose(szUserName);
  Result := True;
end;


begin
  WriteLn(MESSAGE1);
  if InitUserForCSP then
    WriteLn(MESSAGE2)
  else
    WriteLn(MESSAGE3);
end.
