{This document is Copyright  1997-98 Project JEDI.
Visit the JEDI home page at: http://www.delphi-jedi.org/}

{---------------------------------------------------------
Copyright (c) 1996-98 Massimo Maria Ghisalberti
CopyRight (c) 1996-98 MAx!
CopyRight (c) 1996-98 Objects Built for you! (O.B.you!)
Internet EMail: obyou@tin.it

name:           SignFile.dpr
date:           08/18/1998
version:        1 beta revision 1.

note: use at own risk.
      If you use this code, please mention O.B.you!
      somewhere in your program
---------------------------------------------------------
REVISION:

---------------------------------------------------------
File Signature


The SIGNFILE sample is a console application that signs files. Files
signed with this sample can be later verified with the VERIFILE sample.

Note that the INITUSER sample (or equivalent) must be run prior to running
these samples, in order to create a key container for the default user.

Usage
-----

The SIGNFILE sample is run from the command line as follows:

    signfile <source file> <signature file> <description>

The <source file> argument specifies the filename of the file to be signed,
and the <signature file> argument specifies the filename of the file in which
to place the signature data. The <description> argument specifies a textual
description of the data being signed. This can consist of empty quotes ("")
if no description is required. See CryptSignHash in the online documentation
for more information on signatures and description strings.

The VERIFILE sample is run with the same arguments as SIGNFILE. If the
contents of the source file, signature file, or description string has
changed in any way from when the file was originally signed, the program
will fail.

Exercises for the Reader
------------------------

1. These samples would be more useful if the signature public key was stored
   in the signature file along with the signature data. This public key would
   need to be wrapped in a certificate (also known as a credential) to protect
   it from unauthorized modification. If this were to be done, then the data
   file and its signature could be verified by anyone, even if the original
   key pair used to produce the signature is destroyed.

2. The SIGNFILE sample could be easily modified such that the description data
   is stored in the signature file (in a non-encrypted format) along with the
   signature data. The VERIFILE sample could then read the description from
   the signature file, instead of requiring it as a command line argument.
-------------------------------------------------------------------------------}


{$APPTYPE CONSOLE}

program signf;
uses
  Windows, WinCrypt;
  
const
  MESSAGE1 = '------------------------------------------------------------' + #13#10 +
             'VerF version 1.0.' + #13#10+
             '------------------------------------------------------------' + #13#10 +
             'Utility for digital signature.'                               + #13#10;
  MESSAGE2 = 'USAGE: VerF <source file> <signature file> <description>' +#13#10;
  MESSAGE3 = 'Digital signature successfully verified.';

  MESSAGE4 = 'The <source file> argument specifies the filename of the file to be signed,'+ #13#10 +
             'and the <signature file> argument specifies the filename of the file in which'+ #13#10 +
             'to place the signature data. The <description> argument specifies a textual'+ #13#10 +
             'description of the data being signed. This can consist of empty quotes ("")'+ #13#10 +
             'if no description is required.';

var
   hSourceFile    :THANDLE;
   hSignatureFile :THANDLE;
   Endof          :boolean;
   hProv          :HCRYPTPROV;
   hHash          :HCRYPTHASH;
   pbSignature    :PBYTE;
   dwSignatureLen :DWORD;
   hSigPublicKey  :HCRYPTKEY;
   pbBuffer       :PBYTE;
   dwBufferLen    :DWORD;
   dwCount        :DWORD;

function EOF(hFile :THANDLE; BytesRead :DWORD):boolean;
begin
  Result := false;
  if (hFile and  BytesRead) = 0 then
    Result := true;
end;

procedure CleanAll;
begin
  // Close Files.
  if boolean(hSourceFile) then
    CloseHandle(hSourceFile);
  if boolean(hSignatureFile) then
    CloseHandle(hSignatureFile);
  // Free memory.
  if boolean(pbSignature) then
    Dispose(pbSignature);
  if boolean(pbBuffer) then
    Dispose(pbBuffer);
  // Release crypto handles.
  if boolean(hHash) then
    CryptDestroyHash(hHash);
  if boolean(hProv) then
    CryptReleaseContext(hProv, 0);
end;

procedure _Exit(msg :string);
begin
  WriteLn(msg);
  CleanAll;
  Halt;
end;

procedure VerifyFile(szSourceFile, szSignatureFile, szDescription :string);
begin
  {Try to acquire handle to CSP.}
  if not CryptAcquireContext(@hProv,nil,nil,PROV_RSA_FULL,0) then
    _Exit('Error during CryptAcquireContext!');
    
  // Determine number of bytes to hash at once.
  dwBufferLen := 1000;
  
  GetMem(pbBuffer, dwBufferLen);
  if (pbBuffer = nil) then
    _Exit('Out of memory!!!');
    
  // Open source file.
  hSourceFile := CreateFile(PCHAR(szSourceFile),
                            GENERIC_READ,
                            FILE_SHARE_READ,
                            nil,
                            OPEN_EXISTING,
                            FILE_ATTRIBUTE_NORMAL,
                            0);
  // Create a hash object.
  if not(CryptCreateHash(hProv, CALG_MD5, 0, 0, @hHash)) then
    _Exit('Error during CryptCreateHash!');
  repeat
    // Read up to 'dwBufferLen' bytes from source file.
    if not boolean(ReadFile(hSourceFile,pbBuffer^,dwBufferLen,dwCount,nil)) then
      _Exit('Error reading Plaintext!');

    Endof := Eof(hSourceFile,dwCount);
    // Add data to hash object.
    if not (CryptHashData(hHash, pbBuffer, dwCount, 0)) then
      _Exit('Error during CryptHashData!');

  until Endof;//Eof(hSourceFile);
  // Close source file.
  CloseHandle(hSourceFile);
  // Free 'pbBuffer' memory.
  Dispose(pbBuffer);
  pbBuffer := nil;

  // Determine size of signature.
  dwSignatureLen := 0;
  if not(CryptSignHash(hHash, AT_SIGNATURE, nil, 0, nil, @dwSignatureLen)) then
    _Exit('Error during CryptSignHash 1!');
    
  GetMem(pbSignature, dwSignatureLen);
  if (pbSignature = nil) then
    _Exit('Out of memory!!!');

  // Sign hash object (with signature key).
  hSignatureFile := CreateFile(PCHAR(szSignatureFile),
                               GENERIC_READ,
                               FILE_SHARE_READ,
                               nil,
                               OPEN_EXISTING,
                               FILE_ATTRIBUTE_NORMAL,
                               0);
  if hSignatureFile = INVALID_HANDLE_VALUE then
    _Exit('Signature file do not exist!.');
    
  dwCount := 0;
  if not boolean(ReadFile(hSignatureFile,pbSignature^,dwSignatureLen,dwCount,nil)) then
    _Exit('Error writing signature!');

  // Get handle to signature key.
  if not(CryptGetUserKey(hProv, AT_SIGNATURE, @hSigPublicKey)) then
      _Exit('Error during CryptGetUserKey!');

  // Verify signature.
    if not(CryptVerifySignature(hHash, pbSignature, dwSignatureLen,
                                hSigPublicKey, PChar(szDescription), 0)) then
      if(GetLastError = NTE_BAD_SIGNATURE) then
        begin
	  _Exit('Signature did not match!');
        end
      else
	_Exit('Error during CryptVerifySignature!');

  CloseHandle(hSignatureFile);
  // Free 'pbSignature' memory.
  Dispose(pbSignature);
  pbSignature := nil;
  CleanAll;
  WriteLn(MESSAGE3);
end;

begin
  if not(ParamCount = 3) then
    begin
      WriteLn(MESSAGE1);
      WriteLn(MESSAGE2);
      WriteLn(MESSAGE4);
    end
  else
    VerifyFile(ParamStr(1),ParamStr(2),ParamStr(3));
end.